@@ -1,36 +1,30 @@
 #!/usr/bin/env python3
-
 import argparse
-import os
 import json
+import os
 import statistics
-
-from typing import Any, Dict, List, Mapping, Sequence, Tuple
+from pathlib import Path
+from typing import Any
+from typing import Dict
+from typing import List
+from typing import Mapping
+from typing import Sequence
+from typing import Tuple
 
 from potodo import __version__
 from potodo._github import get_reservation_list
-from potodo._po_file import PoFileStats, get_po_files_from_repo
+from potodo._po_file import get_po_stats_from_repo
+from potodo._po_file import PoFileStats
+
 
 # TODO: Sort the functions (maybe in different files ?
 
 
-def initialize_arguments(
-    above: int, below: int, offline: bool, hide_reserved: bool, repo_path: str
-) -> Tuple[int, int, Mapping[str, str]]:
-    """Will initialize the arguments as necessary
+def get_issue_reservations(
+    offline: bool, hide_reserved: bool, repo_path: Path
+) -> Dict[str, Tuple[Any, Any]]:
+    """Retrieve info about reservation if needed.
     """
-    if not above:
-        # If above isn't specified, then print all files above 0% (all of them)
-        above = 0
-    if not below:
-        # If below isn't specified, then print all files below 100% (all of them)
-        below = 100
-
-    if above and below:
-        if below < above:
-            # If above and below are specified and that below is superior to above,
-            # raise an error
-            raise ValueError("Below must be inferior to above")
 
     if not offline and not hide_reserved:
         # If the reservations are to be displayed, then get them
@@ -38,7 +32,25 @@ def initialize_arguments(
     else:
         # Otherwise, an empty list will do the trick
         issue_reservations = {}
-    return above, below, issue_reservations
+    return issue_reservations
+
+
+def check_args(
+    path: str, exclude: List[str], below: int, above: int, **kwargs: Any
+) -> Mapping[str, Any]:
+    # If below is lower than above, raise an error
+    if below < above:
+        raise ValueError("'below' must be greater than 'above'.")
+
+    # If no path is specified, use current directory
+    if not path:
+        path = os.getcwd()
+
+    # Convert strings to `Path` objects and make them absolute
+    return {
+        "path": Path(path).resolve(),
+        "exclude": [Path(path).resolve() for path in exclude],
+    }
 
 
 def print_dir_stats(
@@ -79,15 +91,19 @@ def add_dir_stats(
 
 
 def exec_potodo(
-    path: str,
-    exclude: List[str],
+    path: Path,
+    exclude: List[Path],
     above: int,
     below: int,
-    fuzzy: bool,
+    only_fuzzy: bool,
     offline: bool,
     hide_reserved: bool,
     counts: bool,
     json_format: bool,
+    exclude_fuzzy: bool,
+    exclude_reserved: bool,
+    only_reserved: bool,
+    show_reservation_dates: bool,
 ) -> None:
     """
     Will run everything based on the given parameters
@@ -96,20 +112,23 @@ def exec_potodo(
     :param exclude: folders or files to be ignored
     :param above: The above threshold
     :param below: The below threshold
-    :param fuzzy: Should only fuzzies be printed
+    :param only_fuzzy: Should only fuzzies be printed
     :param offline: Will not connect to internet
     :param hide_reserved: Will not show the reserved files
     :param counts: Render list with counts not percentage
     :param json_format: Format output as JSON.
+    :param exclude_fuzzy: Will exclude files with fuzzies in output.
+    :param exclude_reserved: Will print out only files that aren't reserved
+    :param only_reserved: Will print only reserved fils
+    :param show_reservation_dates: Will show the reservation dates
     """
 
     # Initialize the arguments
-    above, below, issue_reservations = initialize_arguments(
-        above, below, offline, hide_reserved, path
-    )
+    issue_reservations = get_issue_reservations(offline, hide_reserved, path)
 
-    # Get a dict with the directory name and all po files.
-    po_files_and_dirs = get_po_files_from_repo(path, exclude)
+    # Dict whose keys are directory names and values are
+    # stats for each file within the directory.
+    po_files_and_dirs = get_po_stats_from_repo(path, exclude)
 
     dir_stats: List[Any] = []
     for directory_name, po_files in sorted(po_files_and_dirs.items()):
@@ -120,7 +139,9 @@ def exec_potodo(
 
         for po_file in sorted(po_files):
             # For each file in those files from that directory
-            if not fuzzy or po_file.fuzzy_entries:
+            if not only_fuzzy or po_file.fuzzy_entries:
+                if exclude_fuzzy and po_file.fuzzy_entries:
+                    continue
                 buffer_add(
                     buffer,
                     folder_stats,
@@ -131,6 +152,9 @@ def exec_potodo(
                     below,
                     counts,
                     json_format,
+                    exclude_reserved,
+                    only_reserved,
+                    show_reservation_dates,
                 )
 
         # Once all files have been processed, print the dir and the files
@@ -150,11 +174,14 @@ def buffer_add(
     folder_stats: List[int],
     printed_list: List[bool],
     po_file_stats: PoFileStats,
-    issue_reservations: Mapping[str, str],
+    issue_reservations: Dict[str, Tuple[Any, Any]],
     above: int,
     below: int,
     counts: bool,
     json_format: bool,
+    exclude_reserved: bool,
+    only_reserved: bool,
+    show_reservation_dates: bool,
 ) -> None:
     """Will add to the buffer the information to print about the file is
     the file isn't translated entirely or above or below requested
@@ -189,9 +216,16 @@ def buffer_add(
     po_file_size = po_file_stats.po_file_size
     # percentage of the file already translated
     percent_translated = po_file_stats.percent_translated
+    
     # `reserved by` if the file is reserved
+    reserved_by, reservation_date = issue_reservations.get(
+        po_file_stats.filename_dir.lower(), (None, None)
+    )
     # unless the offline/hide_reservation are enabled
-    reserved_by = issue_reservations.get(po_file_stats.filename_dir.lower(), None)
+    if exclude_reserved and reserved_by:
+        return
+    if only_reserved and not reserved_by:
+        return
 
     directory = po_file_stats.directory
     filename = po_file_stats.filename
@@ -208,6 +242,7 @@ def buffer_add(
             translated=translated_nb,
             percent_translated=percent_translated,
             reserved_by=reserved_by,
+            reservation_date=reservation_date,
         )
 
         buffer.append(d)
@@ -227,6 +262,8 @@ def buffer_add(
 
         if reserved_by is not None:
             s += f", réservé par {reserved_by}"
+            if show_reservation_dates:
+                s += f" ({reservation_date})"
 
         buffer.append(s)
 
@@ -257,6 +294,7 @@ def main() -> None:
     parser.add_argument(
         "-a",
         "--above",
+        default=0,
         metavar="X",
         type=int,
         help="list all TODOs above given X%% completion",
@@ -265,13 +303,18 @@ def main() -> None:
     parser.add_argument(
         "-b",
         "--below",
+        default=100,
         metavar="X",
         type=int,
         help="list all TODOs below given X%% completion",
     )
 
     parser.add_argument(
-        "-f", "--fuzzy", action="store_true", help="print only files marked as fuzzys",
+        "-f",
+        "--only-fuzzy",
+        dest="only_fuzzy",
+        action="store_true",
+        help="print only files marked as fuzzys",
     )
 
     parser.add_argument(
@@ -302,17 +345,52 @@ def main() -> None:
         "--json",
         action="store_true",
         dest="json_format",
-        help="format output as JSON.",
+        help="format output as JSON",
+    )
+
+    parser.add_argument(
+        "--exclude-fuzzy",
+        action="store_true",
+        dest="exclude_fuzzy",
+        help="select only files without fuzzy entries",
+    )
+
+    parser.add_argument(
+        "--exclude-reserved",
+        action="store_true",
+        dest="exclude_reserved",
+        help="select only files that aren't reserved",
+    )
+
+    parser.add_argument(
+        "--only-reserved",
+        action="store_true",
+        dest="only_reserved",
+        help="select only only reserved files",
+    )
+
+    parser.add_argument(
+        "--show-reservation-dates",
+        action="store_true",
+        dest="show_reservation_dates",
+        help="show issue creation dates",
     )
 
     parser.add_argument(
         "--version", action="version", version="%(prog)s " + __version__
     )
 
-    args = parser.parse_args()
+    # Initialize args and check consistency
+    args = vars(parser.parse_args())
+    args.update(check_args(**args))
 
-    # If no path is specified, use current directory
-    if not args.path:
-        args.path = os.getcwd()
+    if args.get("exclude_fuzzy") and args.get("only_fuzzy"):
+        print("Cannot pass --exclude-fuzzy and --only-fuzzy at the same time")
+        exit(1)
 
-    exec_potodo(**vars(args))
+    if args.get("exclude_reserved") and args.get("only_reserved"):
+        print("Cannot pass --exclude-reserved and --only-reserved at the same time")
+        exit(1)
+
+    # Launch the processing itself
+    exec_potodo(**args)
